unit VoiceCommandAPIU;

//Uses the Voice Command API as defined in the JEDI Speech import unit

interface

uses
  Speech,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls;

type
  TfrmVoiceCommandAPI = class(TForm)
    lstProgress: TListBox;
    GroupBox1: TGroupBox;
    ProgressBar: TProgressBar;
    lblMinVU: TLabel;
    lblMaxVU: TLabel;
    lblVU: TLabel;
    chkEnabled: TCheckBox;
    chkAwake: TCheckBox;
    btnAbout: TButton;
    btnGeneral: TButton;
    btnLexicon: TButton;
    btnTrainGeneral: TButton;
    btnTrainMic: TButton;
    procedure FormCreate(Sender: TObject);
    procedure chkEnabledClick(Sender: TObject);
    procedure chkAwakeClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnGeneralClick(Sender: TObject);
    procedure btnLexiconClick(Sender: TObject);
    procedure btnTrainGeneralClick(Sender: TObject);
    procedure btnTrainMicClick(Sender: TObject);
  private
    VoiceCmd: IVoiceCmd;
    CmdAttrs: IVCmdAttributes;
    CmdMenu: IVCmdMenu;
    CmdDlgs: IVCmdDialogs;
    procedure CreateCommandMenu;
  public
    procedure Log(const Msg: String); overload;
    procedure Log(const Msg: String; const Args: array of const); overload;
  end;

  TVCmdNotifySink = class(TInterfacedObject, IVCmdNotifySink)
  private
    FForm: TfrmVoiceCommandAPI;
  protected
    function CommandRecognize(dwID: DWORD; pvCmdName: PVCmdNameA; pdwFlags: PDWORD;
      dwActionSize: DWORD; pAction: pointer; dwNumLists: DWORD;
      pszListValues: PAnsiChar; pszCommand: PAnsiChar): HResult; stdcall;
    function CommandOther(pName: PVCmdNameA; pszCommand: PAnsiChar): HResult; stdcall;
    function CommandStart: HResult; stdcall;
    function MenuActivate(pName: PVCmdNameA; bActive: BOOL): HResult; stdcall;
    function UtteranceBegin: HResult; stdcall;
    function UtteranceEnd: HResult; stdcall;
    function VUMeter(wLevel: WORD): HResult; stdcall;
    function AttribChanged(dwAttribute: DWORD): HResult; stdcall;
    function Interference(dwType: DWORD): HResult; stdcall;
  public
    constructor Create(Form: TfrmVoiceCommandAPI);
  end;

var
  frmVoiceCommandAPI: TfrmVoiceCommandAPI;

implementation

{$R *.dfm}

uses
  ComObj;

{$ifndef CONDITIONALEXPRESSIONS}
function BoolToStr(B: Boolean; UseBoolStrs: Boolean = False): string;
const
  cSimpleBoolStrs: array[Boolean] of String = ('0', '-1');
  cBoolStrs: array[Boolean] of String = ('False', 'True');
begin
  if UseBoolStrs then
    Result := cBoolStrs[B]
  else
    Result := cSimpleBoolStrs[B];
end;
{$endif}

procedure TfrmVoiceCommandAPI.Log(const Msg: String);
begin
  if not Assigned(lstProgress) then
    Exit;
  lstProgress.Items.Add(Msg);
  lstProgress.ItemIndex := lstProgress.Items.Count - 1
end;

procedure TfrmVoiceCommandAPI.Log(const Msg: String; const Args: array of const);
begin
  Log(Format(Msg, Args))
end;

procedure TfrmVoiceCommandAPI.FormCreate(Sender: TObject);
var
  Enabled, Awake: DWord;
begin
  VoiceCmd := CreateComObject(CLSID_VCmd) as IVoiceCmd;
  CmdAttrs := VoiceCmd as IVCmdAttributes;
  OleCheck(VoiceCmd.Register(nil, TVCmdNotifySink.Create(Self),
    IVCmdNotifySink, VCMDRF_ALLMESSAGES, nil));
  CreateCommandMenu;
  CmdAttrs.EnabledGet(Enabled);
  chkEnabled.Checked := Bool(Enabled);
  CmdAttrs.AwakeStateGet(Awake);
  chkAwake.Checked := Bool(Awake);
  CmdDlgs := VoiceCmd as IVCmdDialogs;
end;

procedure TfrmVoiceCommandAPI.CreateCommandMenu;

  procedure AddCommand(ID: Integer; const Command, Category, Description: String);
  var
    MemReq: Integer;
    CmdCommand: PVCmdCommand;
    SData: TSData;
    Dest: PChar;
    CmdStart: DWord;
  begin
    MemReq := SizeOf(TVCmdCommand) + Succ(Length(Command)) +
      Succ(Length(Category)) + Succ(Length(Description));
    CmdCommand := AllocMem(MemReq);
    try
      SData.pData := CmdCommand;
      SData.dwSize := MemReq;
      CmdCommand.dwSize := MemReq;
      CmdCommand.dwID := ID;
      //Uncomment the following line to request command verification
      //CmdCommand.dwFlags := VCMDCMD_VERIFY;

      Dest := PChar(CmdCommand) + SizeOf(TVCmdCommand);
      CmdCommand.dwCommand := SizeOf(TVCmdCommand);
      StrCopy(Dest, PChar(Command));

      Inc(Dest, Succ(Length(Command)));
      CmdCommand.dwCategory := SizeOf(TVCmdCommand) + DWord(Succ(Length(Command)));
      StrCopy(Dest, PChar(Category));

      Inc(Dest, Succ(Length(Category)));
      CmdCommand.dwDescription := CmdCommand.dwCommand + DWord(Succ(Length(Category)));
      StrCopy(Dest, PChar(Description));

      OleCheck(CmdMenu.Add(1, SData, CmdStart));
    finally
      FreeMem(CmdCommand);
    end;
  end;

var
  VCMDName: TVCMDName;
begin
  StrPCopy(VCMDName.szApplication, ExtractFileName(Application.ExeName));
  StrPCopy(VCMDName.szState, 'Main');
  OleCheck(VoiceCmd.MenuCreate(
    @VCMDName, nil, VCMDMC_CREATE_TEMP, CmdMenu));
  AddCommand(1, 'Red', 'FormColour', 'Change form colour to red');
  AddCommand(2, 'Green', 'FormColour', 'Change form colour to green');
  AddCommand(3, 'Blue', 'FormColour', 'Change form colour to blue');
  //Command menu is local to this form - pass 0 as the 1st arg to make it global
  //Command menu is an awake menu. Pass VWGFLAG_ASLEEP as the 2nd arg to make it a sleep menu
  OleCheck(CmdMenu.Activate(Handle, 0));
end;

{ TVCmdNotifySink }

constructor TVCmdNotifySink.Create(Form: TfrmVoiceCommandAPI);
begin
  inherited Create;
  FForm := Form
end;

function TVCmdNotifySink.AttribChanged(dwAttribute: DWORD): HResult;
var
  S: String;
  Enabled, Awake: DWord;
begin
  Result := S_OK;
  if dwAttribute <> 0 then
  begin
    if dwAttribute and IVCNSAC_AUTOGAINENABLE > 0 then
      S := 'automatic gain, ';
    if dwAttribute and IVCNSAC_ENABLED > 0 then
    begin
      S := S + 'enabled, ';
      OleCheck(FForm.CmdAttrs.EnabledGet(Enabled));
      FForm.chkEnabled.Checked := Bool(Enabled);
    end;
    if dwAttribute and IVCNSAC_AWAKE > 0 then
    begin
      S := S + 'awake, ';
      OleCheck(FForm.CmdAttrs.AwakeStateGet(Awake));
      FForm.chkAwake.Checked := Bool(Awake);
    end;
    if dwAttribute and IVCNSAC_DEVICE > 0 then
      S := S + 'audio device, ';
    if dwAttribute and IVCNSAC_MICROPHONE > 0 then
      S := S + 'current microphone, ';
    if dwAttribute and IVCNSAC_SPEAKER > 0 then
      S := S + 'speaker, ';
    if dwAttribute and IVCNSAC_SRMODE > 0 then
      S := S + 'SR mode, ';
    if dwAttribute and IVCNSAC_THRESHOLD > 0 then
      S := S + 'threshold, ';
    if dwAttribute and IVCNSAC_ORIGINAPP > 0 then
      S := S + 'from this app';
  end
  else
    S := 'none';
  FForm.Log('Attribute changed: ' + S)
end;

function TVCmdNotifySink.CommandOther(pName: PVCmdNameA;
  pszCommand: PAnsiChar): HResult;
begin
  Result := S_OK;
  FForm.Log('Other command: %s, app: %s, state: %s',
    [pszCommand, pName.szApplication, pName.szState])
end;

//Note that the pdwFlags parameter should be declared:
//  dwFlags: DWord
//Do not treat it like a pointer - instead use:
//  if DWord(pdwFlags) = VCMDCMD_VERIFY then ...
function TVCmdNotifySink.CommandRecognize(dwID: DWORD;
  pvCmdName: PVCmdNameA; pdwFlags: PDWORD; dwActionSize: DWORD;
  pAction: pointer; dwNumLists: DWORD; pszListValues,
  pszCommand: PAnsiChar): HResult;
begin
  Result := S_OK;
  FForm.Log('Command: app = %s, state = %s, cmd = %s, id = %d',
    [pvCmdName.szApplication, pvCmdName.szState, pszCommand, dwId]);
  //If command verification might be required, handle it like this
  //if DWord(pdwFlags) = VCMDCMD_VERIFY then ...
  case dwID of
    1: FForm.Color := clRed;
    2: FForm.Color := clGreen;
    3: FForm.Color := clBlue;
  end
end;

function TVCmdNotifySink.CommandStart: HResult;
begin
  Result := S_OK;
  FForm.Log('Command started')
end;

function TVCmdNotifySink.Interference(dwType: DWORD): HResult;
var
  S: String;
begin
  Result := S_OK;
  case dwType of
    SRMSGINT_NOISE:              S := 'background noise too high';
    SRMSGINT_NOSIGNAL:           S := 'engine cannot detect a signal (mic unplugged?)';
    SRMSGINT_TOOLOUD:            S := 'speaker is too loud; recognition results may be degraded';
    SRMSGINT_TOOQUIET:           S := 'speaker is too quiet; recognition results may be degraded';
    SRMSGINT_AUDIODATA_STOPPED,
    SRMSGINT_IAUDIO_STOPPED:     S := 'engine has stopped receiving audio data from the audio source';
    SRMSGINT_AUDIODATA_STARTED,
    SRMSGINT_IAUDIO_STARTED:     S := 'engine has resumed receiving audio data from the audio source';
  else
    S := Format('type %d', [dwType])
  end;
  FForm.Log('Interference: %s', [S])
end;

function TVCmdNotifySink.MenuActivate(pName: PVCmdNameA;
  bActive: BOOL): HResult;
begin
  Result := S_OK;
  FForm.Log('Menu active: %s, app: %s, state: %s',
    [BoolToStr(bActive, True), pName.szApplication, pName.szState])
end;

function TVCmdNotifySink.UtteranceBegin: HResult;
begin
  Result := S_OK;
  FForm.Log('Utterance begun')
end;

function TVCmdNotifySink.UtteranceEnd: HResult;
begin
  Result := S_OK;
  FForm.Log('Utterance ended')
end;

function TVCmdNotifySink.VUMeter(wLevel: WORD): HResult;
begin
  Result := S_OK;
  FForm.ProgressBar.Position := wLevel;
  FForm.lblVU.Caption := IntToStr(wLevel);
end;

procedure TfrmVoiceCommandAPI.chkEnabledClick(Sender: TObject);
begin
  OleCheck(CmdAttrs.EnabledSet(Integer(chkEnabled.Checked)))
end;

procedure TfrmVoiceCommandAPI.chkAwakeClick(Sender: TObject);
begin
  OleCheck(CmdAttrs.AwakeStateSet(Integer(chkAwake.Checked)))
end;

procedure TfrmVoiceCommandAPI.btnAboutClick(Sender: TObject);
begin
  OleCheck(CmdDlgs.AboutDlg(Handle, nil))
end;

procedure TfrmVoiceCommandAPI.btnGeneralClick(Sender: TObject);
begin
  OleCheck(CmdDlgs.GeneralDlg(Handle, nil))
end;

procedure TfrmVoiceCommandAPI.btnLexiconClick(Sender: TObject);
begin
  OleCheck(CmdDlgs.LexiconDlg(Handle, nil))
end;

procedure TfrmVoiceCommandAPI.btnTrainGeneralClick(Sender: TObject);
begin
  OleCheck(CmdDlgs.TrainGeneralDlg(Handle, nil))
end;

procedure TfrmVoiceCommandAPI.btnTrainMicClick(Sender: TObject);
begin
  OleCheck(CmdDlgs.TrainMicDlg(Handle, nil))
end;

end.
