unit VoiceDictationAPIU;

//Uses the Voice Dictation API as defined in the JEDI Speech import unit

interface

uses
  Speech,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, Menus;

type
  TfrmVoiceDictationAPI = class(TForm)
    lstProgress: TListBox;
    GroupBox1: TGroupBox;
    ProgressBar: TProgressBar;
    lblMinVU: TLabel;
    lblMaxVU: TLabel;
    lblVU: TLabel;
    memText: TMemo;
    lstHypotheses: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    PopupMenu1: TPopupMenu;
    Showbuiltincommands1: TMenuItem;
    btnAbout: TButton;
    btnGeneral: TButton;
    btnLexicon: TButton;
    btnTrainGeneral: TButton;
    btnTrainMic: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Showbuiltincommands1Click(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnGeneralClick(Sender: TObject);
    procedure btnLexiconClick(Sender: TObject);
    procedure btnTrainGeneralClick(Sender: TObject);
    procedure btnTrainMicClick(Sender: TObject);
  private
    VoiceDct: IVoiceDictation;
    DctAttrs: IVDctAttributes;
    DctDlgs: IVDctDialogs;
  public
    procedure Log(const Msg: String); overload;
    procedure Log(const Msg: String; const Args: array of const); overload;
  end;

  TVDctNotifySink = class(TInterfacedObject, IVDctNotifySink)
  private
    FForm: TfrmVoiceDictationAPI;
    FPhraseDone: Boolean;
    function PhraseToStr(pSRPhrase: PSRPhraseA): String;
  protected
    function CommandBuiltIn(pszCommand: PAnsiChar): HResult; stdcall;
    function CommandOther(pszCommand: PAnsiChar): HResult; stdcall;
    function CommandRecognize(dwID: DWord; pdwFlags: PDWord; dwActionSize: DWORD;
      pAction: Pointer; pszCommand: PAnsiChar): HResult; stdcall;
    function TextSelChanged: HResult; stdcall;
    function TextChanged(dwReason: DWORD): HResult; stdcall;
    function TextBookmarkChanged(dwID: DWORD): HResult; stdcall;
    function PhraseStart: HResult; stdcall;
    function PhraseFinish(dwFlags: DWORD; pSRPhrase: PSRPhraseA): HResult; stdcall;
    function PhraseHypothesis(dwFlags: DWORD; pSRPhrase: PSRPhraseA): HResult; stdcall;
    function UtteranceBegin: HResult; stdcall;
    function UtteranceEnd: HResult; stdcall;
    function VUMeter(wLevel: WORD): HResult; stdcall;
    function AttribChanged(dwAttribute: DWORD): HResult; stdcall;
    function Interference(dwType: DWORD): HResult; stdcall;
    function Training(dwTrain: DWORD): HResult; stdcall;
    function Dictating(pszApp: PAnsiChar; fDictating: BOOL): HResult; stdcall;
  public
    constructor Create(Form: TfrmVoiceDictationAPI);
  end;

var
  frmVoiceDictationAPI: TfrmVoiceDictationAPI;

implementation

{$R *.dfm}

uses
  ComObj, ActiveX;

{$ifndef CONDITIONALEXPRESSIONS}
function BoolToStr(B: Boolean; UseBoolStrs: Boolean = False): string;
const
  cSimpleBoolStrs: array[Boolean] of String = ('0', '-1');
  cBoolStrs: array[Boolean] of String = ('False', 'True');
begin
  if UseBoolStrs then
    Result := cBoolStrs[B]
  else
    Result := cSimpleBoolStrs[B];
end;
{$endif}

{ TfrmVoiceDictationAPI }

procedure TfrmVoiceDictationAPI.Log(const Msg: String);
begin
  if not Assigned(lstProgress) then
    Exit;
  lstProgress.Items.Add(Msg);
  lstProgress.ItemIndex := lstProgress.Items.Count - 1
end;

procedure TfrmVoiceDictationAPI.Log(const Msg: String; const Args: array of const);
begin
  Log(Format(Msg, Args))
end;

procedure TfrmVoiceDictationAPI.FormCreate(Sender: TObject);
begin
  VoiceDct := CreateComObject(CLSID_VDct) as IVoiceDictation;
  OleCheck(VoiceDct.Register(
    PChar(ExtractFileName(Application.ExeName)), nil, nil, nil,
    TVDctNotifySink.Create(Self), IVDctNotifySink, VCMDRF_ALLMESSAGES));
  DctDlgs := VoiceDct as IVDctDialogs;
  DctAttrs := VoiceDct as IVDctAttributes;
  OleCheck(DctAttrs.SpeakerSet('blong'));
  OleCheck(DctAttrs.ModeSet(VSRMODE_CMDANDDCT));
  OleCheck(VoiceDct.Activate(Handle));
end;

{ TVDctNotifySink }

constructor TVDctNotifySink.Create(Form: TfrmVoiceDictationAPI);
begin
  inherited Create;
  FForm := Form
end;

function TVDctNotifySink.AttribChanged(dwAttribute: DWORD): HResult;
var
  S: String;
begin
  Result := S_OK;
  if dwAttribute <> 0 then
  begin
    if dwAttribute and IVCNSAC_AUTOGAINENABLE > 0 then
      S := 'automatic gain, ';
    if dwAttribute and IVCNSAC_ENABLED > 0 then
      S := S + 'enabled, ';
    if dwAttribute and IVCNSAC_AWAKE > 0 then
      S := S + 'awake, ';
    if dwAttribute and IVCNSAC_DEVICE > 0 then
      S := S + 'audio device, ';
    if dwAttribute and IVCNSAC_MICROPHONE > 0 then
      S := S + 'current microphone, ';
    if dwAttribute and IVCNSAC_SPEAKER > 0 then
      S := S + 'speaker, ';
    if dwAttribute and IVCNSAC_SRMODE > 0 then
      S := S + 'SR mode, ';
    if dwAttribute and IVCNSAC_THRESHOLD > 0 then
      S := S + 'threshold, ';
    if dwAttribute and IVCNSAC_ORIGINAPP > 0 then
      S := S + 'from this app';
  end
  else
    S := 'none';
  FForm.Log('Attribute changed: ' + S)
end;

function TVDctNotifySink.CommandBuiltIn(pszCommand: PAnsiChar): HResult;
begin
  Result := S_OK;
  FForm.Log('Command built-in: %s', [pszCommand])
end;

function TVDctNotifySink.CommandOther(pszCommand: PAnsiChar): HResult;
begin
  Result := S_OK;
  FForm.Log('Other command: %s', [pszCommand])
end;

function TVDctNotifySink.CommandRecognize(dwID: DWord; pdwFlags: PDWord;
  dwActionSize: DWORD; pAction: Pointer; pszCommand: PAnsiChar): HResult;
begin
  Result := S_OK;
  FForm.Log('Command: cmd = %s, id = %d', [pszCommand, dwId]);
end;

function TVDctNotifySink.Dictating(pszApp: PAnsiChar;
  fDictating: BOOL): HResult;
begin
  Result := S_OK;
  FForm.Log('Dictating: %s, app = %s', [BoolToStr(fDictating, True), pszApp])
end;

function TVDctNotifySink.Interference(dwType: DWORD): HResult;
var
  S: String;
begin
  Result := S_OK;
  case dwType of
    SRMSGINT_NOISE:              S := 'background noise too high';
    SRMSGINT_NOSIGNAL:           S := 'engine cannot detect a signal (mic unplugged?)';
    SRMSGINT_TOOLOUD:            S := 'speaker is too loud; recognition results may be degraded';
    SRMSGINT_TOOQUIET:           S := 'speaker is too quiet; recognition results may be degraded';
    SRMSGINT_AUDIODATA_STOPPED,
    SRMSGINT_IAUDIO_STOPPED:     S := 'engine has stopped receiving audio data from the audio source';
    SRMSGINT_AUDIODATA_STARTED,
    SRMSGINT_IAUDIO_STARTED:     S := 'engine has resumed receiving audio data from the audio source';
  else
    S := Format('type %d', [dwType])
  end;
  FForm.Log('Interference: %s', [S])
end;

function TVDctNotifySink.PhraseToStr(pSRPhrase: PSRPhraseA): String;
var
  ToGo: Integer;
  PSRW: PSRWord;
begin
  Result := '';
  if pSRPhrase = nil then
    Exit;
  ToGo := pSRPhrase.dwSize - SizeOf(pSRPhrase.dwSize);
  PSRW := @pSRPhrase.abWords;
  while ToGo > 0 do
  begin
    Result := Result + PChar(@PSRW.szWord) + #32;
    Dec(ToGo, PSRW.dwSize);
    Inc(PChar(PSRW), PSRW.dwSize)
  end;
end;

function TVDctNotifySink.PhraseFinish(dwFlags: DWORD;
  pSRPhrase: PSRPhraseA): HResult;
begin
  Result := S_OK;
  if dwFlags and ISRNOTEFIN_RECOGNIZED > 0 then
  begin
    FForm.Log('PhraseFinish: %s', [PhraseToStr(pSRPhrase)]);
    FForm.memText.SelText := PhraseToStr(pSRPhrase);
  end
  else
    FForm.Log('PhraseFinish: unrecognised phrase');
  //Since PhraseStart never seems to trigger, this will
  //clear the old hypothese list on the first new hypothesis
  FPhraseDone := True
end;

function TVDctNotifySink.PhraseHypothesis(dwFlags: DWORD;
  pSRPhrase: PSRPhraseA): HResult;
begin
  Result := S_OK;
  //Since PhraseStart never seems to trigger, this will
  //clear the old hypotheses list on the first new hypothesis
  if FPhraseDone then
  begin
    FForm.lstHypotheses.Clear;
    FPhraseDone := False
  end;
  if dwFlags and ISRNOTEFIN_RECOGNIZED > 0 then
  begin
    FForm.lstHypotheses.Items.Add(PhraseToStr(pSRPhrase));
    FForm.lstHypotheses.ItemIndex := FForm.lstHypotheses.Items.Count - 1
  end
  else
    FForm.Log('PhraseHypothesis: unrecognised phrase');
end;

//This notification is never actually called
function TVDctNotifySink.PhraseStart: HResult;
begin
  Result := S_OK;
  FForm.Log('Phrase started')
end;

function TVDctNotifySink.TextBookmarkChanged(dwID: DWORD): HResult;
begin
  Result := S_OK;
  FForm.Log('TextBookmarkChanged: %d', [dwID])
end;

function TVDctNotifySink.TextChanged(dwReason: DWORD): HResult;
var
  S: String;
const
  VDCT_TEXTMOVED = 8;
begin
  Result := S_OK;
  if dwReason <> 0 then
  begin
    if dwReason and VDCT_TEXTADDED > 0 then
      S := 'text added, ';
    if dwReason and VDCT_TEXTREMOVED > 0 then
      S := 'text removed, ';
    if dwReason and VDCT_TEXTREPLACED > 0 then
      S := 'text replaced, ';
    if dwReason and VDCT_TEXTMOVED > 0 then
      S := 'text moved, ';
  end;
  FForm.Log('TextChanged: %s', [S])
end;

function TVDctNotifySink.TextSelChanged: HResult;
begin
  Result := S_OK;
  FForm.Log('TextSelChanged')
end;

function TVDctNotifySink.Training(dwTrain: DWORD): HResult;
var
  S: String;
const
  SRGNSTTRAIN_MICROPHONE = 1;
  SRGNSTTRAIN_GRAMMAR    = 2;
  SRGNSTTRAIN_GENERAL    = 3;
begin
  Result := S_OK;
  if dwTrain <> 0 then
    case dwTrain of
      SRGNSTTRAIN_MICROPHONE: S := 'for the current microphone';
      SRGNSTTRAIN_GRAMMAR: S := 'for the current grammar';
      SRGNSTTRAIN_GENERAL: S := 'in general';
    end;
  FForm.Log('Training required %s', [S])
end;

function TVDctNotifySink.UtteranceBegin: HResult;
begin
  Result := S_OK;
  FForm.Log('Utterance begun')
end;

function TVDctNotifySink.UtteranceEnd: HResult;
begin
  Result := S_OK;
  FForm.Log('Utterance ended')
end;

function TVDctNotifySink.VUMeter(wLevel: WORD): HResult;
begin
  Result := S_OK;
  FForm.ProgressBar.Position := wLevel;
  FForm.lblVU.Caption := IntToStr(wLevel);
end;

procedure TfrmVoiceDictationAPI.Showbuiltincommands1Click(Sender: TObject);
var
  Buf: PChar;
  BufSize: DWord;
begin
  //Get built-in commands
  with VoiceDct as IVDctCommandsBuiltIn do
  begin
    TextGet(Buf, BufSize);
    memText.Text := Buf;
    CoTaskMemFree(Buf);
  end
end;

procedure TfrmVoiceDictationAPI.btnAboutClick(Sender: TObject);
begin
  OleCheck(DctDlgs.AboutDlg(Handle, nil))
end;

procedure TfrmVoiceDictationAPI.btnGeneralClick(Sender: TObject);
begin
  OleCheck(DctDlgs.GeneralDlg(Handle, nil))
end;

procedure TfrmVoiceDictationAPI.btnLexiconClick(Sender: TObject);
begin
  OleCheck(DctDlgs.LexiconDlg(Handle, nil))
end;

procedure TfrmVoiceDictationAPI.btnTrainGeneralClick(Sender: TObject);
begin
  //Note the API for general training and mic training are the wrong way round
  OleCheck(DctDlgs.TrainMicDlg(Handle, nil))
end;

procedure TfrmVoiceDictationAPI.btnTrainMicClick(Sender: TObject);
begin
  //Note the API for general training and mic training are the wrong way round
  OleCheck(DctDlgs.TrainGeneralDlg(Handle, nil))
end;

end.
