unit VoiceTextAPIU;

//Uses the Voice Text API as defined in the JEDI Speech import unit

interface

uses
  Speech,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ImgList, StdCtrls, ComCtrls, ToolWin;

type
  TfrmVoiceTextAPI = class(TForm)
    lstProgress: TListBox;
    Label1: TLabel;
    ImageList1: TImageList;
    reText: TRichEdit;
    ToolBar1: TToolBar;
    btnPlay: TToolButton;
    btnPause: TToolButton;
    btnStop: TToolButton;
    Label7: TLabel;
    btnAbout: TButton;
    btnGeneral: TButton;
    btnLexicon: TButton;
    btnTranslate: TButton;
    Label8: TLabel;
    memEnginePhonemes: TMemo;
    chkEnabled: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnGeneralClick(Sender: TObject);
    procedure btnLexiconClick(Sender: TObject);
    procedure btnTranslateClick(Sender: TObject);
    procedure chkEnabledClick(Sender: TObject);
  private
    VoiceText: IVoiceText;
    TxtAttrs: IVTxtAttributes;
    TxtDlgs: IVTxtDialogs;
    BeenPaused: Boolean;
  public
    procedure Log(const Msg: String); overload;
    procedure Log(const Msg: String; const Args: array of const); overload;
  end;

  TVTxtNotifySink = class(TInterfacedObject, IVTxtNotifySink)
  private
    FForm: TfrmVoiceTextAPI;
  protected
    function AttribChanged(dwAttribute: DWORD): HResult; stdcall;
    function Visual(cIPAPhoneme: WideChar; cEnginePhoneme: AnsiChar; // cIPAPhoneme is WideChar even in ANSI compile
      dwHints: DWORD; pTTSMouth: PTTSMOUTH): HResult; stdcall;
    function Speak(pszText: PAnsiChar; pszApplication: PAnsiChar; dwType: DWORD): HResult; stdcall;
    function SpeakingStarted: HResult; stdcall;
    function SpeakingDone: HResult; stdcall;
  public
    constructor Create(Form: TfrmVoiceTextAPI);
  end;

var
  frmVoiceTextAPI: TfrmVoiceTextAPI;

implementation

{$R *.dfm}

uses
  ComObj;

{ TfrmVoiceTextAPI }

procedure TfrmVoiceTextAPI.Log(const Msg: String);
begin
  if not Assigned(lstProgress) then
    Exit;
  lstProgress.Items.Add(Msg);
  lstProgress.ItemIndex := lstProgress.Items.Count - 1
end;

procedure TfrmVoiceTextAPI.Log(const Msg: String; const Args: array of const);
begin
  Log(Format(Msg, Args))
end;

procedure TfrmVoiceTextAPI.FormCreate(Sender: TObject);
var
  Enabled: DWord;
begin
  SendMessage(lstProgress.Handle, LB_SETHORIZONTALEXTENT, Width, 0);
  VoiceText := CreateComObject(CLSID_VTxt) as IVoiceText;
  OleCheck(VoiceText.Register(nil, PChar(Application.ExeName),
    TVTxtNotifySink.Create(Self), IVTxtNotifySink, VTXTF_ALLMESSAGES, nil));
  TxtAttrs := VoiceText as IVTxtAttributes;
  OleCheck(TxtAttrs.EnabledGet(Enabled));
  chkEnabled.Checked := Bool(Enabled);
  TxtDlgs := VoiceText as IVTxtDialogs;
end;

procedure TfrmVoiceTextAPI.btnPlayClick(Sender: TObject);
begin
  if not BeenPaused then
    OleCheck(VoiceText.Speak(PChar(reText.Text), VTXTST_STATEMENT or VTXTSP_NORMAL, nil))
  else
  begin
    OleCheck(VoiceText.AudioResume);
    BeenPaused := False
  end;
end;

procedure TfrmVoiceTextAPI.btnPauseClick(Sender: TObject);
begin
  if VoiceText.AudioPause = NOERROR then
    BeenPaused := True
end;

procedure TfrmVoiceTextAPI.btnStopClick(Sender: TObject);
begin
  OleCheck(VoiceText.StopSpeaking);
  BeenPaused := False;
end;

procedure TfrmVoiceTextAPI.btnAboutClick(Sender: TObject);
begin
  OleCheck(TxtDlgs.AboutDlg(Handle, nil))
end;

procedure TfrmVoiceTextAPI.btnGeneralClick(Sender: TObject);
begin
  OleCheck(TxtDlgs.GeneralDlg(Handle, nil))
end;

procedure TfrmVoiceTextAPI.btnLexiconClick(Sender: TObject);
begin
  OleCheck(TxtDlgs.LexiconDlg(Handle, nil))
end;

procedure TfrmVoiceTextAPI.btnTranslateClick(Sender: TObject);
begin
  OleCheck(TxtDlgs.TranslateDlg(Handle, nil))
end;

{ TVTxtNotifySink }

constructor TVTxtNotifySink.Create(Form: TfrmVoiceTextAPI);
begin
  inherited Create;
  FForm := Form
end;

function TVTxtNotifySink.AttribChanged(dwAttribute: DWORD): HResult;
var
  S: String;
begin
  Result := S_OK;
  case dwAttribute of
    TTSNSAC_REALTIME : S := 'Realtime';
    TTSNSAC_PITCH    : S := 'Pitch';
    TTSNSAC_SPEED    : S := 'Speed';
    TTSNSAC_VOLUME   : S := 'Volume';
  else
    S := 'unknown'
  end;
  FForm.Log('Engine Event AttribChanged: %s changed', [S]);
end;

function TVTxtNotifySink.Speak(pszText, pszApplication: PAnsiChar;
  dwType: DWORD): HResult;
begin
  Result := S_OK;
  FForm.Log('Speak: application %s', [pszApplication]);
  FForm.memEnginePhonemes.Clear
end;

function TVTxtNotifySink.SpeakingDone: HResult;
begin
  Result := S_OK;
  FForm.Log('SpeakingDone');
end;

function TVTxtNotifySink.SpeakingStarted: HResult;
begin
  Result := S_OK;
  FForm.Log('SpeakingStarted');
end;

function TVTxtNotifySink.Visual(cIPAPhoneme: WideChar;
  cEnginePhoneme: AnsiChar; dwHints: DWORD; pTTSMouth: PTTSMOUTH): HResult;
var
  Hint: String;
begin
  Result := S_OK;
  Hint := '';
  if dwHints <> 0 then
  begin
    if dwHints and TTSNSHINT_QUESTION <> 0 then
      Hint := 'Question ';
    if dwHints and TTSNSHINT_STATEMENT <> 0 then
      Hint := Hint + 'Statement ';
    if dwHints and TTSNSHINT_COMMAND <> 0 then
      Hint := Hint + 'Command ';
    if dwHints and TTSNSHINT_EXCLAMATION <> 0 then
      Hint := Hint + 'Exclamation ';
    if dwHints and TTSNSHINT_EMPHASIS <> 0 then
      Hint := Hint + 'Emphasis';
  end
  else
    Hint := 'none';
  FForm.Log('Visual: hint = %s', [Hint]);
  if cEnginePhoneme <> #32 then
    FForm.memEnginePhonemes.Text := FForm.memEnginePhonemes.Text + cEnginePhoneme
end;

procedure TfrmVoiceTextAPI.chkEnabledClick(Sender: TObject);
begin
  OleCheck(TxtAttrs.EnabledSet(DWord(chkEnabled.Checked)))
end;

end.
